package com.chelvc.framework.common.model;

import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.deser.ContextualDeserializer;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.Getter;
import lombok.NonNull;

/**
 * 自定义枚举抽象类
 *
 * @author Woody
 * @date 2025/5/17
 */
@Getter
@JsonSerialize(using = Enumeration.JacksonSerializer.class)
@JsonDeserialize(using = Enumeration.JacksonDeserializer.class)
public abstract class Enumeration implements Serializable {
    /**
     * 枚举对象JSON序列化处理器
     */
    static class JacksonSerializer extends JsonSerializer<Enumeration> {
        @Override
        public void serialize(Enumeration value, JsonGenerator generator, SerializerProvider provider)
                throws IOException {
            generator.writeString(value.getCode());
        }
    }

    /**
     * 枚举对象JSON反序列化处理器
     */
    static class JacksonDeserializer extends JsonDeserializer<Enumeration> implements ContextualDeserializer {
        private final Class<? extends Enumeration> type;

        public JacksonDeserializer() {
            this(null);
        }

        public JacksonDeserializer(Class<? extends Enumeration> type) {
            this.type = type;
        }

        @Override
        public Enumeration deserialize(JsonParser parser, DeserializationContext context) throws IOException {
            return Enumeration.parse(this.type, parser.getValueAsString());
        }

        @Override
        @SuppressWarnings("unchecked")
        public JsonDeserializer<?> createContextual(DeserializationContext context, BeanProperty property) {
            return new JacksonDeserializer((Class<? extends Enumeration>) context.getContextualType().getRawClass());
        }
    }

    /**
     * 枚举对象/编码/排序/是否动态映射表
     */
    private static final Map<Class<?>, List<BiPair<String, Integer, Boolean>>> VALUES = Maps.newConcurrentMap();

    /**
     * 枚举对象/编码/实例映射表
     */
    private static final Map<Class<?>, Map<String, Enumeration>> INSTANCES = Maps.newConcurrentMap();

    /**
     * 枚举编码
     */
    private final String code;

    /**
     * 排序数字
     */
    private final int ordinal;

    /**
     * 是否动态
     */
    private final boolean dynamic;

    /**
     * 描述信息
     */
    private final String description;

    protected Enumeration() {
        this(null, null);
    }

    protected Enumeration(Integer ordinal) {
        this(ordinal, null);
    }

    protected Enumeration(String description) {
        this(null, description);
    }

    protected Enumeration(Integer ordinal, String description) {
        Class<?> clazz = this.getClass();
        synchronized (clazz.getName().intern()) {
            List<BiPair<String, Integer, Boolean>> values = VALUES.get(clazz);
            if (ObjectUtils.isEmpty(values) && INSTANCES.containsKey(clazz)) {
                throw new IllegalStateException("Invalid enumeration instance");
            } else if (ObjectUtils.isEmpty(values)) {
                values = VALUES.computeIfAbsent(clazz, Enumeration::getValues);
            }
            BiPair<String, Integer, Boolean> pair = values.remove(0);
            this.code = Objects.requireNonNull(pair.getLeft());
            this.ordinal = ObjectUtils.ifNull(ordinal, pair::getMiddle);
            this.dynamic = Boolean.TRUE.equals(pair.getRight());
            if (ObjectUtils.isEmpty(values)) {
                VALUES.remove(clazz);
            }
        }

        this.description = ObjectUtils.ifNull(description, this.code);
        INSTANCES.computeIfAbsent(clazz, k -> Maps.newConcurrentMap()).putIfAbsent(this.code, this);
    }

    /**
     * 生成枚举实例
     *
     * @param type 枚举类型
     * @param <T>  对象类型
     * @return 枚举实例
     */
    @SuppressWarnings("unchecked")
    private static <T extends Enumeration> T generateInstance(Class<T> type) {
        Constructor<?>[] constructors = type.getDeclaredConstructors();
        if (constructors.length > 1) {
            Arrays.sort(constructors, Comparator.comparingInt(Constructor::getParameterCount));
        }
        Constructor<?> constructor = constructors[0];
        Object[] parameters = new Object[constructor.getParameterCount()];
        if (parameters.length > 0) {
            Class<?>[] types = constructor.getParameterTypes();
            for (int i = 0; i < parameters.length; i++) {
                Class<?> clazz = types[i];
                if (clazz == char.class) {
                    parameters[i] = (char) 0;
                } else if (clazz == byte.class) {
                    parameters[i] = (byte) 0;
                } else if (clazz == int.class) {
                    parameters[i] = 0;
                } else if (clazz == short.class) {
                    parameters[i] = (short) 0;
                } else if (clazz == long.class) {
                    parameters[i] = 0L;
                } else if (clazz == float.class) {
                    parameters[i] = 0.0F;
                } else if (clazz == double.class) {
                    parameters[i] = 0.00D;
                } else if (clazz == boolean.class) {
                    parameters[i] = false;
                } else {
                    parameters[i] = null;
                }
            }
        }
        if (!constructor.isAccessible()) {
            constructor.setAccessible(true);
        }
        try {
            return (T) constructor.newInstance(parameters);
        } catch (ReflectiveOperationException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取枚举值，需要返回一个可更新列表
     *
     * @param clazz 枚举对象
     * @return 枚举编码/排序/是否动态信息列表
     */
    private static List<BiPair<String, Integer, Boolean>> getValues(Class<?> clazz) {
        Field[] fields = clazz.getFields();
        List<BiPair<String, Integer, Boolean>> values = Lists.newLinkedList();
        if (ObjectUtils.notEmpty(fields)) {
            for (Field field : fields) {
                Class<?> type = field.getType();
                int modifiers = field.getModifiers();
                if (type == clazz && Modifier.isPublic(modifiers) && Modifier.isStatic(modifiers)
                        && Modifier.isFinal(modifiers)) {
                    values.add(BiPair.of(field.getName(), values.size(), false));
                }
            }
        }
        return values;
    }

    /**
     * 枚举转换
     *
     * @param type  枚举类型
     * @param value 枚举值
     * @param <T>   对象类型
     * @return 枚举实例
     */
    @SuppressWarnings("unchecked")
    public static <T extends Enumeration> T parse(@NonNull Class<T> type, String value) {
        if (StringUtils.isEmpty(value)) {
            return null;
        }

        Enumeration instance;
        Map<String, Enumeration> instances = INSTANCES.get(type);
        if (instances == null || (instance = instances.get(value)) == null) {
            boolean dynamic;
            synchronized (type.getName().intern()) {
                if ((instances = INSTANCES.get(type)) == null) {
                    // 枚举字段尚未初始化
                    List<BiPair<String, Integer, Boolean>> values =
                            VALUES.computeIfAbsent(type, Enumeration::getValues);
                    BiPair<String, Integer, Boolean> found =
                            values.stream().filter(p -> Objects.equals(p.getLeft(), value)).findAny().orElse(null);
                    int ordinal = (dynamic = Objects.isNull(found)) ? 0 : found.getMiddle();
                    values.add(BiPair.of(value, ordinal, dynamic));
                } else if ((instance = instances.get(value)) == null) {
                    // 枚举字段已经初始化
                    VALUES.put(type, Lists.newArrayList(BiPair.of(value, 0, dynamic = true)));
                } else {
                    return (T) instance;
                }

                // 初始化枚举实例
                instance = generateInstance(type);
            }

            // 如果不是动态枚举则重新获取，保证一致性
            if (!dynamic) {
                instance = INSTANCES.get(type).get(value);
            }
        }
        return (T) instance;
    }

    /**
     * 获取枚举实例
     *
     * @param type 枚举类型
     * @param <T>  对象类型
     * @return 枚举编码/实例映射表
     */
    @SuppressWarnings("unchecked")
    public static <T extends Enumeration> Map<String, T> getInstances(@NonNull Class<T> type) {
        Map<String, T> instances = (Map<String, T>) INSTANCES.get(type);
        if (instances == null) {
            synchronized (type.getName().intern()) {
                if ((instances = (Map<String, T>) INSTANCES.get(type)) == null) {
                    // 临时设置一个无效枚举实例，待枚举对象实例初始化完成后移除
                    parse(type, StringUtils.SPACE);
                    instances = (Map<String, T>) INSTANCES.get(type);
                    instances.remove(StringUtils.SPACE);
                }
            }
        }
        return Collections.unmodifiableMap(instances);
    }

    @Override
    public int hashCode() {
        return this.code.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        return obj == this || (obj != null && obj.getClass() == this.getClass()
                && Objects.equals(this.code, ((Enumeration) obj).code));
    }

    @Override
    public String toString() {
        return this.code;
    }
}
