package com.chelvc.framework.common.model;

import java.io.Serializable;
import java.util.function.Function;

import com.chelvc.framework.common.util.NumberUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import lombok.Builder;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.NonNull;

/**
 * 数字范围对象
 *
 * @author Woody
 * @date 2024/10/30
 */
@Getter
@Builder
@EqualsAndHashCode
public final class Range implements Serializable {
    /**
     * 最小值
     */
    private Number min;

    /**
     * 最大值
     */
    private Number max;

    public Range() {
    }

    public Range(Number min, Number max) {
        this.min = min;
        this.max = max;
    }

    /**
     * 将字符串转换成数字范围信息
     *
     * @param text 范围信息字符串格式
     * @return 范围信息实例
     */
    public static Range parse(String text) {
        return StringUtils.decompose(text, (l, r, d) -> {
            if (d < 0) {
                return new Range(l == null ? r : l, r == null ? l : r);
            }
            return new Range(l, r);
        }, number -> number.indexOf('.') > -1 ? Double.parseDouble(number) : Long.parseLong(number));
    }

    /**
     * 判断是否包含指定数字
     *
     * @param number 数字
     * @return true/false
     */
    public boolean contains(Number number) {
        return number != null && (this.min != null || this.max != null)
                && (this.min == null || NumberUtils.compare(this.min, number) <= 0)
                && (this.max == null || NumberUtils.compare(this.max, number) >= 0);
    }

    /**
     * 获取最小值
     *
     * @param function 返回值转换函数
     * @param <R>      返回值类型
     * @return 最小值
     */
    public <R> R getMin(@NonNull Function<Number, R> function) {
        return ObjectUtils.ifNull(this.min, function);
    }

    /**
     * 获取最大值
     *
     * @param function 返回值转换函数
     * @param <R>      返回值类型
     * @return 最大值
     */
    public <R> R getMax(@NonNull Function<Number, R> function) {
        return ObjectUtils.ifNull(this.max, function);
    }

    @Override
    public String toString() {
        StringBuilder buffer = new StringBuilder();
        if (this.min != null) {
            buffer.append(this.min);
        }
        buffer.append('_');
        if (this.max != null) {
            buffer.append(this.max);
        }
        return buffer.toString();
    }
}
